--  Modifiziertes Testfile
--  File: Peak_Detect.VHD
--  Creator: Steffen Manthey :-)

library IEEE;
use IEEE.std_logic_1164.all;

entity Peak_Detect is
    port (
        LOG_L: in STD_LOGIC_VECTOR (3 downto 0);
        LOG_R: in STD_LOGIC_VECTOR (3 downto 0);
        FLAG: in STD_LOGIC;
        MAX_L: out STD_LOGIC_VECTOR (3 downto 0):= "0000";
        MAX_R: out STD_LOGIC_VECTOR (3 downto 0):= "0000";
        PEAK_L: out STD_LOGIC_VECTOR (3 downto 0):= "0000";
        PEAK_R: out STD_LOGIC_VECTOR (3 downto 0):= "0000";
        count_short : out INTEGER range 0 to 2204; 	--dient der Darstellung der Zhlerwerte nach 
        count_long_l: out INTEGER range 0 to 40; 	--auen, zum Testen
        count_long_r: out INTEGER range 0 to 40
    );
end Peak_Detect;

architecture Peak_Detect of Peak_Detect is

 	--Zwischenspeicher fr langfristige Maxima
	Signal peak_left, peak_right: STD_LOGIC_VECTOR (3 downto 0):= "0000";
	--Zwischenspeicher fr kurzfristige Maxima
	Signal max_left, max_right: STD_LOGIC_VECTOR (3 downto 0):= "0000";

	--Wert den den groessten absoluten Pegelwert kleiner gleich -12 db repraesentiert
	Constant twelve_db: STD_LOGIC_VECTOR (3 downto 0):= "1000";
	
	--Zhler fr kurzfristige Maxima
	Signal count_max: INTEGER range 0 to 2204;
	 
	--Zhler fr langfristige Maxima des linken Kanals
	Signal count_peak_l: INTEGER range 0 to 40; 
	--Zhler fr langfristige Maxima des rechten Kanals
	Signal count_peak_r: INTEGER range 0 to 40; 

begin
  
	alles: PROCESS
	
	begin
	
	    --auf fallende Taktflanke warten
		wait until Flag = '0';
		
		--testen ob 0,05s vergangen (2205 * (1/44100)s = 0,05s)
		if count_max = 0 then
		
			--fr die linken Spitzenwerte
			--testen ob inzwischen grerer Spizenwert vorliegt
			if (max_left >= peak_left) AND (max_left > twelve_db) then
				--Zhler zurcksetzen und neuen Wert zuweisen
				count_peak_l <= 0;
				peak_left <= max_left;
			end if;
			
			--Zhler fr die rechten Spitzenwerte
			--testen ob inzwischen grerer Spizenwert vorliegt
			if (max_right >= peak_right) AND (max_right > twelve_db) then
				--Zhler zurcksetzen und neuen Wert zuweisen
				count_peak_r <= 0;
				peak_right <= max_right;
			end if;
		end if;
			
		wait for 5 ns;
			
		if count_max = 0 then
			--Ausgabe des kurzfristigen Maxima
			MAX_L <= max_left;
			MAX_R <= max_right;
			--und Rcksetzen
			max_left  <= LOG_L;
			max_right <= LOG_R;
				
			--Ausgabe des Langfristigen Maxima
			PEAK_L <= peak_left;
			PEAK_R <= peak_right;
				
		else
		
			--Ermittlung des kurzfristigen Maxima
			if LOG_L > max_left then
				max_left <= LOG_L;
			end if;
				
			if LOG_R > max_right then
				max_right <= LOG_R;
			end if;
			
		end if;
	
		--auf steigende Taktflanke warten
		wait until FLAG = '1';
		
		--testen ob 0,05s vergangen (2205 * (1/44100)s = 0,05s)
		if count_max = 15 then 
	
			--rcksetzen des Zhlers auf Null
			count_max <= 0;
			--Erhhen der Zhler fr die langfristigen Maxima
			count_peak_l <= count_peak_l + 1;
			count_peak_r <= count_peak_r + 1;
			
			--fr die linken Spitzenwerte
			--testen ob 2s vergangen (40 * 0,05s = 2s)
			if count_peak_l = 39 then
				peak_left <= "0000";
				count_peak_l <= 0;
			else
				--Erhhen der Zhler fr die langfristigen Maxima
				count_peak_l <= count_peak_l + 1;
			end if;
			
			--Zhler fr die rechten Spitzenwerte
			--testen ob 2s vergangen (40 * 0,05s = 2s)
			if count_peak_r = 39 then
				peak_right <= "0000";
				count_peak_r <= 0;
			else
				--Erhhen der Zhler fr die langfristigen Maxima
				count_peak_r <= count_peak_r + 1;
			end if;
			
		else
			--Erhhen des Zhlers fr das kurzfristige Maxima
			count_max <= count_max + 1;
		end if;
		
	end Process;
	
	count_short <= count_max;
	count_long_l <= count_peak_l;
	count_long_r <= count_peak_r;
				
			
end Peak_Detect;

